searchData={"content_type":"text/markdown","items":[{"doc":"Declarative programming in Elixir\n\nDiesel is a toolkit that helps you build your own DSLs.\n\nExample:\n\n```elixir\ndefmodule Latex.Dsl do\n  use Diesel.Dsl,\n    otp_app: :latex,\n    root: :latex,\n    packages: [...],\n    tags: [\n      :document,\n      :package,\n      :packages,\n      :section,\n      :subsection,\n      ...\n    ],\nend\n\ndefmodule Latex.Pdf do\n  @behaviour Diesel.Generator\n\n  @impl true\n  def generate(_mod, definition) do\n    quote do\n      def to_pdf, do: \"%PDF-1.4 ...\"\n    end\n  end\nend\n\ndefmodule Latex do\n  use Diesel,\n    otp_app: :my_app,\n    dsl: Latex.Dsl,\n    generators: [\n      Latex.Pdf\n    ]\nend\n```\n\nthen we could use it with:\n\n```elixir\ndefmodule MyApp.Paper do\n  use Latex\n\n  latex do\n    document size: \"a4\" do\n      packages [:babel, :graphics]\n\n      section title: \"Introduction\" do\n        subsection title: \"Details\" do\n          ...\n        end\n      end\n    end\n  end\nend\n\niex> MyApp.Paper.to_pdf()\n\"%PDF-1.4 ...\"\n```\n\nDSLs built with Diesel are not sealed: they can be easily extended by via packages and code generators. These can be even defined by other apps, via application environment:\n\n```elixir\nconfig :latex, Latex.Dsl, packages: [ ...]\nconfig :my_app, MyApp.Paper, generators: [...]\n```\n\nPlease take a look at the `Diesel.Dsl` module documentation and also the examples provided in tests.","ref":"Diesel.html","title":"Diesel","type":"behaviour"},{"doc":"","ref":"Diesel.html#c:compile/1","title":"Diesel.compile/1","type":"callback"},{"doc":"","ref":"Diesel.html#c:definition/0","title":"Diesel.definition/0","type":"callback"},{"doc":"","ref":"Diesel.html#t:attrs/0","title":"Diesel.attrs/0","type":"type"},{"doc":"","ref":"Diesel.html#t:element/0","title":"Diesel.element/0","type":"type"},{"doc":"","ref":"Diesel.html#t:tag/0","title":"Diesel.tag/0","type":"type"},{"doc":"Defines the syntax provided by a DSL.\n\nDSLs built with Diesel:\n\n* are documents. They look like HTML.\n* define tags\n* can be compiled\n* can be extended via packages and code generators\n\nSimple usage:\n\n```elixir\ndefmodule Latex.Dsl do\n  use Diesel.Dsl,\n    otp_app: :my_app,\n    root: :latex,\n    tags: [\n      :document,\n      :package,\n      :section,\n      :subsection\n    ]\nend\n```\n\nA DSL can be extended via packages:\n\n```elixir\ndefmodule Latex.Dsl do\n  use Diesel.Dsl,\n    ...\n    packages: [\n      Latex.Dsl.Music\n    ]\nend\n```\n\nPackages allow you to define extra tags, as well as compiler rules.\n\nAdditional packages can be specified via application environment.\n\nThese will be appended to the list of packages already declared in the module:\n\n```elixir\nconfig :latex, Latex.Dsl,\n  packages: [\n    OtherApp.Latex.Dsl.Math\n  ]\n```","ref":"Diesel.Dsl.html","title":"Diesel.Dsl","type":"behaviour"},{"doc":"","ref":"Diesel.Dsl.html#c:compile/2","title":"Diesel.Dsl.compile/2","type":"callback"},{"doc":"","ref":"Diesel.Dsl.html#c:locals_without_parens/0","title":"Diesel.Dsl.locals_without_parens/0","type":"callback"},{"doc":"","ref":"Diesel.Dsl.html#c:root/0","title":"Diesel.Dsl.root/0","type":"callback"},{"doc":"","ref":"Diesel.Dsl.html#c:tags/0","title":"Diesel.Dsl.tags/0","type":"callback"},{"doc":"Generators produce Elixir code based on DSLs, before modules are compiled.","ref":"Diesel.Generator.html","title":"Diesel.Generator","type":"behaviour"},{"doc":"","ref":"Diesel.Generator.html#c:generate/2","title":"Diesel.Generator.generate/2","type":"callback"},{"doc":"Packages extend DSLs by providing new tags.\n\nOptionally, packages can also specify how these tags should be compiled, in order to form a new definition that can then be consumed by code generators.\n\nUsage:\n\n```elixir\ndefmodule Latex.Dsl.Music do\n  use Diesel.Package,\n    tags: [\n      :music,\n      :instrument,\n      :meter\n    ]\n\n  @impl true\n  def compiler do\n    quote do\n      def compile({:music, attrs, children}, ctx) do\n        atttrs = Keyword.put_new(attrs, :indent, \"10mm\")\n        children = compile(children, ctx)\n\n        {:music, attrs, children}\n      end\n    end\n  end\nend\n```\n\nThe code returned by each package via the `compiler/1` function will be injected into the Dsl.\n\nThe `Diesel.Dsl` macro provides with a default implementation for the `compile/2` callback, so that you can conveniently traverse the definition tree.","ref":"Diesel.Package.html","title":"Diesel.Package","type":"behaviour"},{"doc":"","ref":"Diesel.Package.html#c:compiler/0","title":"Diesel.Package.compiler/0","type":"callback"},{"doc":"","ref":"Diesel.Package.html#c:tags/0","title":"Diesel.Package.tags/0","type":"callback"},{"doc":"Liquid templating support for DSLs during compilation.\n\nExample:\n\n```elixir\ndefmodule Paper do\n  use Latex\n\n  latex do\n    document size: \"{{ doc.size }}\" do\n      ...\n    end\n  end\nend\n\niex> Paper.compile(%{doc: %{size: \"a4\"}})\n```","ref":"Diesel.Templating.html","title":"Diesel.Templating","type":"module"},{"doc":"","ref":"Diesel.Templating.html#render!/2","title":"Diesel.Templating.render!/2","type":"function"}]}